% Voir livre Perron et Munger: Analyse des systèmes et matrice d'état et exercices multi-physiques 2022 
% REMARQUE AU LECTEUR: LE MODÈLE SUGGÉRÉ EST UN EXEMPLE ACCADÉMIQUE.  
% TOUTES AUTRES APPLICATIONS EST AU RISQUE DE L'UTILISATEUR

% Problème : Système de freinage ABS 
close all , clear all , clc

%% Parametres du probleme
md = 2;         % Masse du disque [kg]
mr = 15;        % Masse de la roue [kg]
mv = 622.5;     % 1/4 masse de la voiture [kg]
Rd = 0.15;      % Rayon du disque [m]
R = 0.35;       % Rayon de la roue [m]
A = 0.01;       % Surface étrier [m^2]
Pmax = 2500000; % Pression maximale [Pa]
mud = 0.36;     % Coefficient de friction du disque [-]
mur1 = 0.015;   % Coefficient de friction de la roue sans glissement [-]
mur2 = 0.005;   % Coefficient de friction de la roue avec glissement [-]
g = 9.81;       % Accélération gravitationnelle [m/s^2]
I = 1.8825;     % Inertie [kgm^2]
v0 = 100/3.6;   % Vitesse initiale [m/s]


%% Question 1 : Trouver l'equation d'etat
% syms b1 b2 Rd R A Pmax mt g I mud
mt = md + mr + mv;
c = I + mt*R^2;
b1 = -(mud*Rd*R*A*Pmax)/(I + mt*R^2);
b2 = -(mt*g*R^2)/(I + mt*R^2);

matA = [0 1;0 0];    % Matrice d'etat
matB = [0 0;b1 b2];  % Matrice de controle

%% Question 2 : Trouver le resolvant
syms s

Identite = eye(2);          % Matrice identite
Phi = inv(s*Identite-matA); % Resolvant

 %% Quesiton 3 : Trouver G1(s) = x(s)/beta(s) et G2(s) = x(s)/mur(s)
C1 = [1 0];
Gx = C1*Phi*matB;
G1 = Gx(1); % x(s)/beta(s)
G2 = Gx(2); % x(s)/muR(s)
 
C2 = [0 1];
Gv= C2*Phi*matB;
G3 = Gv(1); % v(s)/beta(s)
G4 = Gv(2); % v(s)/muR(s)

%% Question 5 : Trouver x(t) et v(t)
syms t  %beta mur1 v0 mur
beta = 0.9;
mur = mur1;
u = [beta/s;mur/s];
CI = [0; v0];
xS = C1*Phi*(CI+matB*u); % x(s)
xT = ilaplace(xS,t);     % x(t)
vS = C2*Phi*(CI+matB*u); % v(s)
vT = ilaplace(vS,t);     % v(t)

%% Question 6 : Tracer x(t) et v(t) avec beta = 0.9u-1 et mur = 0.015u-1
tf = solve(vT==0);
xfinal = subs(xT,t,tf);
time = linspace(0,tf,1000);
xTaff = subs(xT,t,time);
vTaff = subs(vT,t,time);

figure (1)
hold on
plot(time,xTaff)
xlim([0 5.2])
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position du véhicule en boucle ouverte')
grid on
hold off

figure (2)
hold on
plot(time,vTaff)
xlim([0 5.2])
ylim([0 30])
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse du véhicule en boucle ouverte')
grid on
hold off

%% Question 7-11 : Modèle avec glissement
syms t muf x0 v0 %g
CI = [x0 ; v0];
matB = [0;-g];
u2 = muf/s;
x2S = C1*Phi*(CI+matB*u2);
x2T = ilaplace(x2S,t); % x(t) avec glissement
v2S = C2*Phi*(CI+matB*u2);
v2T = ilaplace(v2S,t); % v(t) avec glissement

% Sans glissement
t1 = linspace(0,1,1000);
muf1 = 0.7;
x01 = 0;
v01 = 100/3.6;

temp = subs(x2T,muf,muf1);
temp1 = subs(temp,x0,x01);
temp2 = subs(temp1,v0,v01);
x2Taff1 = subs(temp2,t,t1);

temp = subs(v2T,muf,muf1);
temp1 = subs(temp,x0,x01);
temp2 = subs(temp1,v0,v01);
v2Taff1 = subs(temp2,t,t1);

% Avec glissement
muf2 = 0.4; % Avec glissement
x02 = double(x2Taff1(end)); % Position initiale du glissement (m)
v02 = double(v2Taff1(end)); % Vitesse initiale du glissement (m/s)

temp = subs(v2T,muf,muf2);
temp1 = subs(temp,x0,x02);
temp2 = subs(temp1,v0,v02);

tf2 = solve(temp2==0,t);   % Résoudre v(t) = 0
t2 = linspace(0,tf2,1000); % Durée d'arrêt après glissement

v2Taff2 = subs(temp2,t,t2);

temp = subs(x2T,muf,muf2);
temp1 = subs(temp,x0,x02);
temp2 = subs(temp1,v0,v02);
x2Taff2 = subs(temp2,t,t2);

t3 = [t1, t1(end)+t2(2:end)]; % Durée totale

figure (3)
hold on
plot(t3,[x2Taff1,x2Taff2(2:end)])
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position du véhicule')
grid on
hold off

figure (4)
hold on
plot(t3,[v2Taff1,v2Taff2(2:end)])
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse du véhicule')
grid on
hold off

%% Évolution de la position des roues
acc1 = -(mud*Rd*R*A*Pmax*beta)/c- mt*g*R^2*mur/c; % a(t) sans glissement
acc1T = linspace(acc1,acc1,1000); 

acc2 = -muf2*g; % a(t) avec glissement
acc2T = linspace(acc2,acc2,1000); 
    
alphaT1 = acc1T/R;
alphaT2 = acc2T*0;

omegaT1 = zeros(1,1000); % Initialisation
thetaT1 = zeros(1,1000); % Initialisation
deltaT = 1/length(acc1T); % Pas de temps (s)
for i = 1:1000
    if i == 1
        omegaT1(i) = vTaff(1)/R; % Sans glissement omega(t) numérique
        thetaT1(i) = 0; % Sans glissement theta(t) numérique
    else
        omegaT1(i) = omegaT1(i-1)+alphaT1(i-1)*deltaT; % Sans glissement omega(t) numérique
        thetaT1(i) = thetaT1(i-1)+omegaT1(i-1)*deltaT; % Sans glissement theta(t) numérique
    end
end
omegaT2 = zeros(1,1000); % Avec glissement omega(t) numérique
thetaT2 = linspace(thetaT1(end),thetaT1(end),1000); % Avec glissement theta(t) numérique

figure (5)
hold on
plot(t3,[thetaT1,thetaT2(2:end)])
xlabel('Temps (s)')
ylabel('Position (rad)')
title('Évolution de la position angulaire des roues')
grid on
hold off

figure (6)
hold on
plot(t3,[omegaT1,omegaT2(2:end)])
xlabel('Temps (s)')
ylabel('Vitesse (rad/s)')
title('Évolution de la vitesse angulaire des roues')
grid on
hold off
          
figure (7)
hold on
plot(t3,[alphaT1,alphaT2(2:end)])
xlabel('Temps (s)')
ylabel('Accélération (rad/s^2)')
title('Évolution de l''accélération angulaire des roues')
grid on
hold off

%% Question 13
syms Gc Gbeta Gmu omegaR muR k1 tau b1 b2 v0 tf t R
Gc = k1 + 1/(tau*s);
Gbeta = b1/s;
Gmu = b2/s;
HS = (Gc*Gbeta)/(1+(Gc*Gbeta));
omegaRS = laplace(-v0*t/(R*tf)+(v0/R));
omegaS = HS*omegaRS + (Gmu/R)*(muR/s);
omegaT = ilaplace(omegaS,t);