% Voir livre Perron et Munger: Analyse des systèmes et matrice d'état et exercices multi-physiques 2022 
% REMARQUE AU LECTEUR: LE MODÈLE SUGGÉRÉ EST UN EXEMPLE ACCADÉMIQUE.  
% TOUTES AUTRES APPLICATIONS EST AU RISQUE DE L'UTILISATEUR

% Problème : Sous-marin
close all , clear all , clc

%% Parametres du probleme
masseS = 24330000; % Masse sous-marin début de plongee (début immersion totale)[kg]
masseF = 14700000; % Masse sous-marin en surface (coque et eau de ballast)[kg] 
Vc = 24330;        % Volume de la coque [m^3]
W = 190;           % Puissance de l'helice en avance [MW]
yMax = 480;        % Profondeur maximale de plongée [m]
L = 170;           % Longueur du sous-marin de forme cylindrique [m]
d = 13.5;          % Maitre-bau du sous-marin (diamètre) [m]
Ax = pi*d^2/4;     % Surface de trainee (maitre-couple) [m^2]
vEau = 1.52e6;     % Viscosite cinématique de l'eau a 4 Degree C [m^2}/s]
vMax = 14.8;       % Vitesse maximale en avance (plongee) [m/s]
rho = 1000;        % Densité constante de l'eau [kg/m^3]
Cxc = 0.81;        % Coefficient global de trainee du sous-marin [1]
FN = 1/2*rho*Cxc*Ax*vMax^2;% Force de poussee maximale de l’helice [N] 
g = 9.81;          % Constante gravitationnelle [m/s^2]
K1 = 1/2*rho*Cxc*Ax*vMax;% Constante pour la force de trainée direction x [kg/s].

%%
%%%%%%% PARTIE A %%%%%%%
%% Question 1 : Trouver l'equation d'etat
% syms a b 
a = -K1/masseS;
b = FN/masseS;

matA = [0 1;0 a]; % Matrice d'etat
matB = [0 ; b];   % Matrice de controle

%% Question 2 : Trouver le resolvant phi(s)
syms s 
I = eye(2);          % Matrice identite
Phi = inv(s*I-matA); % Resolvant

%% Question 3 : Trouver l'expression de x(t)
syms alpha t 
U = alpha/s;
C = [1 0];                         % Matrice d'observation de la vitesse
PositionS = C*Phi*matB*U;          % x(s)
PositionT = ilaplace(PositionS,t); % x(t)

%% Question 4 : Trouver l'expression de v(t)
C = [0 1]; % Matrice d'observation de la vitesse
VitesseS = C*Phi*matB*U; % v(s)
VitesseT = ilaplace(VitesseS,t); % v(t)

%% Question 5 : Tracer les graphiques de position et de vitesse
alphaCommande = [0.1,0.5,1];
time = linspace(0,800,2000); % Vecteur temps [s]

Position1T = subs(PositionT,alpha,alphaCommande(1));
Position1Taff = subs(Position1T,t,time);
Position2T = subs(PositionT,alpha,alphaCommande(2));
Position2Taff = subs(Position2T,t,time);
Position3T = subs(PositionT,alpha,alphaCommande(3));
Position3Taff = subs(Position3T,t,time);

Vitesse1T = subs(VitesseT,alpha,alphaCommande(1));
Vitesse1Taff = subs(Vitesse1T,t,time);
Vitesse2T = subs(VitesseT,alpha,alphaCommande(2));
Vitesse2Taff = subs(Vitesse2T,t,time);
Vitesse3T = subs(VitesseT,alpha,alphaCommande(3));
Vitesse3Taff = subs(Vitesse3T,t,time);


figure (1)
hold on
plot(time,Position1Taff)
plot(time,Position2Taff,'--')
plot(time,Position3Taff,'-.')
xlabel('Temps (s)')
ylabel('Position x (m)')
title('Position du sous-marin')
legend('\alpha=0.1','\alpha=0.5','\alpha=1')
grid on
hold off

figure (2)
hold on
plot(time,Vitesse1Taff)
plot(time,Vitesse2Taff,'--')
plot(time,Vitesse3Taff,'-.')
xlabel('Temps (s)')
ylabel('Vitesse x (m/s)')
title('Vitesse du sous-marin')
legend('\alpha=0.1','\alpha=0.5','\alpha=1')
grid on
hold off

%%
%%%%%%% PARTIE B %%%%%%% 
%% Question 6 : Trouver l'equation d'etat
syms beta

matA = [0 1;0 0]; % Matrice d'etat
matB = [0 ; g]; % Matrice de controle

%% Question 7 : Trouver le resolvant phi(s)
syms s 
I = eye(2); % Matrice identite
Phi = inv(s*I-matA); % Resolvant

%% Question 8 : Trouver l'expression de x(t) et v(t)
Y1 = [0]; % Distance de plongee [m]
Y2 = [0];
Vy1 = [0]; % Vitesse de plongee [m]
Vy2 = [0];
Mpo = [10 20] ; % Débit d'eau variable de la pompe du ballast

for i = 2:length(time-1)
    Vy1 = [Vy1, g*(1-(rho*Vc)/(masseS+Mpo(1)*i))*(time(i)-time(i-1))+Vy1(i-1)];
    Vy2 = [Vy2, g*(1-(rho*Vc)/(masseS+Mpo(2)*i))*(time(i)-time(i-1))+Vy2(i-1)];
    Y1 = [Y1, Vy1(i-1)*(time(i)-time(i-1))+Y1(i-1)];
    Y2 = [Y2, Vy2(i-1)*(time(i)-time(i-1))+Y2(i-1)];
end

%% Question 9 : Tracer les graphiques de position et de vitesse
figure (3)
hold on
plot(time,Y1)
plot(time,Y2,'--')
xlabel('Temps (s)')
ylabel('Position y (m)')
title('Position du sous-marin')
legend('$\dot{m}_{po}$=10 (kg/s)','$\dot{m}_{po}$=20 (kg/s)', 'Interpreter', 'latex')
grid on
hold off

figure (4)
hold on
plot(time,Vy1)
plot(time,Vy2,'--')
xlabel('Temps (s)')
ylabel('Vitesse y (m/s)')
title('Vitesse du sous-marin')
legend('$\dot{m}_{po}$=10 (kg/s)','$\dot{m}_{po}$=20 (kg/s)', 'Interpreter', 'latex')
grid on
hold off