% Voir livre Perron et Munger: Analyse des systèmes et matrice d'état et exercices multi-physiques 2022 
% REMARQUE AU LECTEUR: LE MODÈLE SUGGÉRÉ EST UN EXEMPLE ACCADÉMIQUE.  
% TOUTES AUTRES APPLICATIONS EST AU RISQUE DE L'UTILISATEUR

% Problème : Ascenseur
close all , clear all , clc

%% Parametres du probleme
r = 0.5 ;            % Rayon de la poulie du treuil [m]
mc = 300 ;           % Masse de la cage d'ascenseur vide [kg]
m2 = 300 ;           % Masse du contre poids [kg]
ma = 75 ;            % Masse d'un passager [kg]
n = 10 ;             % Nombre de personne dans la cage [-]
m1 = mc + n*ma ;     % Masse de la cage d'ascenseur pleine [kg]
mp = 200 ;           % Masse de la poulie et câble enroulé [kg]
e = 3 ;              % Hauteur par étage [m]
nn = 30 ;            % Nombre d'étage de la course [-]
tc = 2*nn ;          % Temps de la course [s]
k = 123.23*nn^(-2) ; % Constante proportionnelle (compensateur P) [kg m/s^2]
I = 25 ;             % Moment d'intertie du treuil [kg m^2]
L = e*nn ;           % Longueur de la course [m]
g = 9.81 ;           % Accélération gravitationnelle [m/s^2]
M0 = -r*g*(m2-m1) ;  % Moment pour cage immobile (poids) (compensateur P) [Nm]
M = (2*L/tc^2/r)*I+g*r*(m1-m2) ; ...
                     % Moment d'accélération du treuil (constant pour BO) [Nm]
%%
%%%%%%% PARTIE A %%%%%%%
%% Question 1 : Trouver l'equation d'etat
%syms m1 m2 r I
b1 = (m2-m1)*r/I ;
b2 = 1/I ;

matA = [0 1;0 0]; % Matrice d'etat
matB = [0 0 ;b1 b2]; % Matrice de controle

%% Question 2 : Trouver x(t) avec le resolvant phi(s)
syms s t %g M
Id = eye(2); % Matrice identite
Phi = inv(s*Id-matA); % Resolvant

U = [g/s ; M/s] ;
C = [1 0];          % Matrice d'observation de la position angulaire

thetaS = C*Phi*matB*U;       % theta(s)
thetaT = ilaplace(thetaS,t); % theta(t)
xT = thetaT*r ;  % x(t)
vT = diff(xT,t); % v(t)
aT = diff(vT,t); % a(t)

%% Question 3 : Tracer les graphiques x(t), v(t), a(t)
time = linspace(0,tc,2000); % Vecteur temps [s]
xTaff = subs(xT,t,time);
vTaff = subs(vT,t,time);
aTaff = subs(aT,t,time);

figure (1)
hold on
plot(time,xTaff)
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position de l''ascenseur')
grid on
hold off

figure (2)
hold on
plot(time,vTaff)
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse de l''ascenseur')
grid on
hold off

figure(3)
hold on
yline(double(aTaff(1)),'color',[0, 0.4470, 0.7410])
xlim([0 tc])
ylim([0 2*double(aTaff(1))])
xlabel('Temps (s)')
ylabel('Accélération (m/s^2)')
title('Évolution de l''accélération de l''ascenseur')
grid on
hold off

%% Question 6 : Trouver x(t) selon le schéma de régulation
syms s %k M0 L r I m2 m1 g
G1 = r/(I*s^2);
Gc = k;
G2 = (m2-m1)*(r^2)/(I*s^2);

xS = ((L/2*Gc*G1/s)+(M0*G1/s)+(g*G2/s))/(1+(Gc*G1));
xT = ilaplace(xS,t);
vT = diff(xT);
aT = diff(vT);

xTaff30 = subs(xT,t,time);
vTaff30 = subs(vT,t,time);
aTaff30 = subs(aT,t,time);

figure (4)
hold on
plot(time,xTaff30)
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position de l''ascenseur')
grid on
hold off

figure (5)
hold on
plot(time,vTaff30)
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse de l''ascenseur')
grid on
hold off

figure (6)
hold on
plot(time,aTaff30)
xlabel('Temps (s)')
ylabel('Accélération (m/s^2)')
title('Évolution de l''accélération de l''ascenseur')
grid on
hold off

close all
%%
%%%%%%% PARTIE B %%%%%%%
%% Question 8 : Proposer des valeurs pour les constantes a et b et tracer pour 100 étages
nn = 100 ;            % Nombre d'étage de la course [-]
tc = 2*nn ;           % Temps de la course [s]
L = e*nn ;            % Longueur de la course [m]
      
a = -1/tc*log(0.001) ; % Pour MATLAB log = ln
K = a*nn^-2;
 
time100 = linspace(0,tc,2000) ;  % Vecteur temps pour 100 étages

xTaff100 = L*(1-exp(-a.*time100));  % x(t) pour 100 étages 
vTaff100 = a*L*exp(-a.*time100);    % v(t) pour 100 étages
aTaff100 = -a^2*L*exp(-a.*time100); % a(t) pour 100 étages
 
figure (7)
hold on
plot(time100,xTaff100)
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position de l''ascenseur')
grid on
hold off

figure (8)
hold on
plot(time100,vTaff100)
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse de l''ascenseur')
grid on
hold off

figure (9)
hold on
plot(time100,aTaff100)
xlabel('Temps (s)')
ylabel('Accélération (m/s^2)')
title('Évolution de l''accélération de l''ascenseur')
grid on
hold off

%% Question 9 : Acceptable pour 1-5 étages?
nn = 5 ;             % Nombre d'étage de la course [-]
tc = 2*nn ;          % Temps de la course [s]
L = e*nn ;           % Longueur de la course [m]
      
a = -1/tc*log(0.001) ; % Pour MATLAB log = ln
K = a*nn^-2;
 
time5 = linspace(0,tc,2000) ;  % Vecteur temps pour 5 étages

xTaff5 = L*(1-exp(-a.*time5)); % x(t) pour 5 étages 
vTaff5 = a*L*exp(-a.*time5); % v(t) pour 5 étages
aTaff5 = -a^2*L*exp(-a.*time5); % a(t) pour 5 étages
 
figure (10)
hold on
plot(time5,xTaff5)
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position de l''ascenseur')
grid on
hold off

figure (11)
hold on
plot(time5,vTaff5)
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse de l''ascenseur')
grid on
hold off

figure (12)
hold on
plot(time5,aTaff5)
xlabel('Temps (s)')
ylabel('Accélération (m/s^2)')
title('Évolution de l''accélération de l''ascenseur')
grid on
hold off

%% Question 10 : Tracer x(t), v(t), a(t) et M(t) pour une course sinusoïdale 30 étages
nn = 30 ;           % Nombre d'étage de la course [-]
tc = 2*nn +10 ;      % Temps de la course [s]
L = e*nn ;          % Longueur de la course [m]
      
time30 = linspace(0,tc,2000) ;  % Vecteur temps pour 30 étages

xT = L/2 * (1+sin(pi*t/tc - pi/2));
vT = diff(xT,t);
aT = diff(vT,t);
MT = (aT-(g*r^2*(m2-m1)/I))/(r/I);

xTaff30 = subs(xT,t,time30); 
vTaff30 = subs(vT,t,time30); 
aTaff30 = subs(aT,t,time30); 
MTaff30 = subs(MT,t,time30);
 
figure (13)
hold on
plot(time30,xTaff30)
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position de l''ascenseur')
grid on
hold off

figure (14)
hold on
plot(time30,vTaff30)
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse de l''ascenseur')
grid on
hold off

figure (15)
hold on
plot(time30,aTaff30)
xlabel('Temps (s)')
ylabel('Accélération (m/s^2)')
title('Évolution de l''accélération de l''ascenseur')
grid on
hold off

figure (16)
hold on
plot(time30,MTaff30)
xlabel('Temps (s)')
ylabel('Moment d''accélération (Nm)')
title('Évolution de M(t) du treuil de l''ascenseur')
grid on
hold off

%% Question 11 : Tracer x(t), v(t), a(t) et M(t) pour une course sinusoïdale 1 étage
nn = 1 ;            % Nombre d'étage de la course [-]
tc = 2*nn+10 ;      % Temps de la course [s]
L = e*nn ;          % Longueur de la course [m]
      
time1 = linspace(0,tc,2000) ;  % Vecteur temps pour 1 étage

xT = L/2 * (1+sin(pi*t/tc - pi/2));
vT = diff(xT,t);
aT = diff(vT,t);
MT = (aT-(g*r^2*(m2-m1)/I))/(r/I);

xTaff1 = subs(xT,t,time1); 
vTaff1 = subs(vT,t,time1); 
aTaff1 = subs(aT,t,time1); 
MTaff1 = subs(MT,t,time1);
 
figure (17)
hold on
plot(time1,xTaff1)
xlabel('Temps (s)')
ylabel('Position (m)')
title('Évolution de la position de l''ascenseur')
grid on
hold off

figure (18)
hold on
plot(time1,vTaff1)
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Évolution de la vitesse de l''ascenseur')
grid on
hold off

figure (19)
hold on
plot(time1,aTaff1)
xlabel('Temps (s)')
ylabel('Accélération (m/s^2)')
title('Évolution de l''accélération de l''ascenseur')
grid on
hold off

figure (20)
hold on
plot(time1,MTaff1)
xlabel('Temps (s)')
ylabel('Moment d''accélération (Nm)')
title('Évolution de M(t) du treuil de l''ascenseur')
grid on
hold off