% Voir livre Perron et Munger: Analyse des systèmes et matrice d'état et exercices multi-physiques 2022 
% REMARQUE AU LECTEUR: LE MODÈLE SUGGÉRÉ EST UN EXEMPLE ACCADÉMIQUE.  
% TOUTES AUTRES APPLICATIONS EST AU RISQUE DE L'UTILISATEUR

% Problème : Module SN5 de SpaceX
close all , clear all , clc

%% Parametres du probleme
m = 93000;      % Masse du module [kg]
Fmoy = 1004400; % Force de poussée moyenne [N]
Fmax = 2200000; % Force de poussée maximale [N]
g = 9.81;       % Accélération gravitationnelle [m/s^2]

%% Question 1 : Trouver l'equation d'etat
syms L I %a1 b1 b2 Fmoy m 
a1 = Fmoy/m;
b1 = 1/m;
b2 = -L*Fmoy/I;

matA = [0 1 0 0 0 0; 0 0 0 0 a1 0; 0 0 0 1 0 0; 0 0 0 0 -a1 0; ...
        0 0 0 0 0 1; 0 0 0 0 0 0]; % Matrice d'etat
matB = [0 0 0 0; 0 a1 0 b1; 0 0 0 0; b1 -a1 -1 0; 0 0 0 0; 0 b2 0 0]; % Matrice de controle

%% Question 2 : Trouver le résolvant
syms s t
I = eye(6); % Matrice identite
Phi = inv(s*I-matA); % Resolvant

%%
%%%%%%%%%% Boucle ouverte (Cas 1 : test altitude maximale)
%% Question 3 : Trouver x(s) et z(s) avec le résolvant
syms F beta V %g
Cx = [1 0 0 0 0 0];    % Matrice observation x
Cz = [0 0 1 0 0 0];    % Matrice observation z
u = [F; beta; g; V];   % Vecteur des entrées

xS = Cx*Phi*matB*u;
zS = Cz*Phi*matB*u;

%% Question 4 : trouver x(t) et z(t) avec le résolvant
syms val t
Fs = laplace(Fmoy*(heaviside(val)-heaviside(val-10)),s);
uS1 = [Fs;0;g/s;0];
xS1 = Cx*Phi*matB*uS1;
zS1 = Cz*Phi*matB*uS1;
zT1 = ilaplace(zS1,t);
vzT1 = diff(zT1,t);

%% Question 5 : Trouver l'altitude maximale
syms t

uS = [Fmoy/s;0;g/s;0];
xS = Cx*Phi*matB*uS;
zS = Cz*Phi*matB*uS;
zT = ilaplace(zS,t);
vzT = diff(zT,t);

h1 = subs(zT,t,10);
vzTaltitudeMax = subs(vzT,t,10);
vzT10 = subs(vzTaltitudeMax ,t,10);     % Vitesse après la poussée
h2 = vzT10^2/(2*g);                     % Position après que le module atteigne 0 m/s
ht = h1 + h2;                           % Hauteur totale

syms t
sol1 = solve(vzT10-g*t==0);       % Temps pour atteindre vitesse nulle
sol2 = solve(ht - g*t^2/2==0,t);  % Temps de descente
tTot = sol1 + sol2(2) + 10 ;      % Temps de la course

time = linspace(0,tTot,1000);
zTaff = subs(zT1,t,time);

figure(1)
hold on
plot(time,zTaff)
xlabel('Temps (s)')
ylabel('Altitude (m)')
title('Altitude du module en fonction du temps')
grid on
hold off

%%
%%%%%%%%%% Boucle ouverte (Cas 2 : Altitude stable à 150, analytique)
%% Question 6 : Trouver des valeurs pour C et k; tracer v(t) et z(t)
k1 = 0.2;
k2 = 0.255;
k3 = 0.3;
dT = 0.01; % Pas de temps
time = (0:dT:40);  % Vecteur temps
% vZ1 = g.*time.*exp(-k1*time); % Vz(t)
% vZ2 = g.*time.*exp(-k2*time);
% vZ3 = g.*time.*exp(-k3*time);
% z1 = 0; % Position initiale
% z2 = 0;
% z3 = 0;
% for i = 2:length(time) % Dérivée numérique
%   z1 = [z1 (z1(i-1)+vZ1(i)*dT)];
%   z2 = [z2 (z2(i-1)+vZ2(i)*dT)];
%   z3 = [z3 (z3(i-1)+vZ3(i)*dT)];
% end   

alpha = 1;
syms t k
z = (-g*t/k)*exp(-k*t) - g/k^2*exp(-k*t)+ alpha*g/k^2; % 
z1temp = subs(z,k,k1);
z2temp = subs(z,k,k2);
z3temp = subs(z,k,k3);

z1 = subs(z1temp,t,time);
z2 = subs(z2temp,t,time);
z3 = subs(z3temp,t,time);

vz = diff(z,t);
vz1temp = subs(vz,k,k1);
vz2temp = subs(vz,k,k2);
vz3temp = subs(vz,k,k3);

vZ1 = subs(vz1temp,t,time);
vZ2 = subs(vz2temp,t,time);
vZ3 = subs(vz3temp,t,time);

figure (2)
hold on
plot(time,z1)
plot(time,z2,'--')
plot(time,z3,'-.')
xlabel('Temps (s)')
ylabel('Altitude (m)')
title('Altitude du module en boucle ouverte (analytique')
legend('k = 0.2','k = 0.255','k = 0.3')
grid on
hold off

figure (3)
hold on
plot(time,vZ1)
plot(time,vZ2,'--')
plot(time,vZ3,'-.')
xlabel('Temps (s)')
ylabel('Vitesse (m/s)')
title('Vitesse verticale du module en boucle ouverte (analytique)')
legend('k = 0.2','k = 0.255','k = 0.3')
grid on
hold off

%% Question 7 : Trouver et tracer l'expression de la force de pousée F(t)
F = g*m*(1-k2.*time).*exp(-k2*time)+m*g ;

figure (4)
hold on
plot(time,F/Fmax)
xlabel('Temps (s)')
ylabel('Force de poussée (% de la force du moteur)')
title('Évolution de la force de poussée en fonction du temps')
grid on
hold off

%% Question 8 : Trouver z(s) par addition des entrées
syms G1 G2 G3 g m Zr K tau s t

G1 = K-1/(tau*s);
G2 = 1/(m*s^2);
G3 = (g*m/s);
z1 = (Zr/s*G1*G2)/(1+G1*G2);
z2 = (-G3*G2)/(1+G1*G2);
zS = z1 + z2; 

%% Question 9-11
K = 93; m = 93000 ; g = 9.81 ; tau = 1.6723e-4 ;Zr = 150;
syms z(t)
ode = diff(z,t,3) + K/m*diff(z,t)-z(t)/(tau*m) == (K*Zr/m)-g;
cond1 = z(0) == 0;
cond2 = z(9999999999999999999) == 150;
conds = [cond1 cond2];
zSolve(t) = dsolve(ode,conds);

time = linspace(0,40,1000);
C1 = 0; C2 = 50; C3 = 150;
z1 = 150.0 - 150.*exp(-0.2.*time).*cos(0.348.*time)+C1.*exp(-0.2.*time).*sin(0.348.*time);
z2 = 150.0 - 150.*exp(-0.2.*time).*cos(0.348.*time)+C2.*exp(-0.2.*time).*sin(0.348.*time);
z3 = 150.0 - 150.*exp(-0.2.*time).*cos(0.348.*time)+C3.*exp(-0.2.*time).*sin(0.348.*time);


figure (5)
hold on
plot(time,z1)
plot(time,z2,'--')
plot(time,z3,'-.')
xlabel('Temps (s)')
ylabel('Altitude (m)')
title('Altitude du module en boucle fermée')
legend('C_3 = 0','C_3 = 50','C_3 = 150')
grid on
hold off

%% Question 12-15
syms z(t) syms C1 C3
tau = 1/(0.008*m);
ode = diff(z,t,3) - K/m*diff(z,t)+z(t)/(tau*m) == 0;
zSolve(t) = dsolve(ode);
C2 = 0; 
z(t) =  C1*exp(-0.202*t) + C2*exp(0.101*t)*cos(0.172*t) - 1.0*C3*exp(0.101*t)*sin(0.172*t);
cond1 = z(0) == 150; % Donc C1 = 150
cond2 = z(inf) == 0; % Donc C3 = 0
z =  150.*exp(-0.202.*time);

figure (6)
hold on
plot(time,z)
xlabel('Temps (s)')
ylabel('Altitude (m)')
title('Atterrissage du module en boucle fermée')
grid on
hold off

%% Question 16
time2 = time+40;

figure (7)
hold on
plot([time,time2],[z1,z])
xlabel('Temps (s)')
ylabel('Altitude (m)')
title('Altitude du module en fonction du temps')
grid on
hold off